ClassierAddonListDB = ClassierAddonListDB or {}

local ADDON_NAME, ns = ...
local ADDON_LOADED = nil
local L = setmetatable(ns.L or {}, { __index = function(t, k) t[k] = k return k	end,})
local iconsGlobal = {}
local iconsClass = {}
local iconsMemory = {}

--local AddonList_Update = AddonList_Update
local character = nil
local characterX = nil
local class = nil
local color = nil

local function IsAddonEnabled(index)
	local result = nil
	if index and type(index) == "number" and class and type("class") == "string" and characterX and type("characterX") == "string" then
		 local name, title, notes, loadable, reason, security = GetAddOnInfo(index)
		 result = ((name == ADDON_NAME) or (ClassierAddonListDB.EnabledGlobal[name]) or (ClassierAddonListDB.EnabledClass[class][name]) or (ClassierAddonListDB.EnabledChar[characterX][name])) --or security == "SECURE"
		 --print(index, result)
	end
	return result	
end

local function IsAddonEnabledChar(index)
	local result = nil
	if index and type(index) == "number" and class and type("class") == "string" and characterX and type("characterX") == "string" then
		 local name, title, notes, loadable, reason, security = GetAddOnInfo(index)
		 result = ((name == ADDON_NAME) or (ClassierAddonListDB.EnabledChar[characterX][name])) --or security == "SECURE"
	end
	return result	
end

local function IsAddonEnabledClass(index)
	local result = nil
	if index and type(index) == "number" and class and type("class") == "string" and characterX and type("characterX") == "string" then
		 local name, title, notes, loadable, reason, security = GetAddOnInfo(index)
		 result = ((name == ADDON_NAME) or (ClassierAddonListDB.EnabledClass[class][name])) --or security == "SECURE"
	end
	return result	
end

local function IsAddonEnabledGlobal(index)
	local result = nil
	if index and type(index) == "number" and class and type("class") == "string" and characterX and type("characterX") == "string" then
		 local name, title, notes, loadable, reason, security = GetAddOnInfo(index)
		 result = ((name == ADDON_NAME) or (ClassierAddonListDB.EnabledGlobal[name])) --or security == "SECURE"
	end
	return result	
end

local function SetAddonEnabledGlobal(index, value)
	 if index and type(index) == "number" and class and type("class") == "string" and characterX and type("characterX") == "string" then
		 local name, title, notes, loadable, reason, security = GetAddOnInfo(index)
		 if name ~= ADDON_NAME then --and security == "INSECURE"
			 ClassierAddonListDB.EnabledChar[characterX][name] = nil
			 ClassierAddonListDB.EnabledClass[class][name] = nil
			 ClassierAddonListDB.EnabledGlobal[name] = value and true or nil
		 end
	 end	 
end

local function SetAddonEnabledClass(index, value)
	 if index and type(index) == "number" and class and type("class") == "string" and characterX and type("characterX") == "string" then
		 local name, title, notes, loadable, reason, security = GetAddOnInfo(index)
		 if name ~= ADDON_NAME then --and security == "INSECURE"
			 ClassierAddonListDB.EnabledChar[characterX][name] = nil
			 ClassierAddonListDB.EnabledGlobal[name] = nil
			 ClassierAddonListDB.EnabledClass[class][name] = value and true or nil
		 end
	 end	 
end

local function SetAddonEnabledChar(index, value)
	 if index and type(index) == "number" and class and type("class") == "string" and characterX and type("characterX") == "string" then
		 local name, title, notes, loadable, reason, security = GetAddOnInfo(index)
		 if name ~= ADDON_NAME then --and security == "INSECURE"
			 ClassierAddonListDB.EnabledGlobal[name] = nil
			 ClassierAddonListDB.EnabledClass[class][name] = nil
			 ClassierAddonListDB.EnabledChar[characterX][name] = value and true or nil
		 end
	 end	 
end

local function CheckAddonDependencies(...)
	for i = 1, select("#", ...) do
		local dep = select(i, ...)
		if GetAddOnEnableState(character, dep) == 0 then
			return false
		end
	end
	return true
end

local AddonList_Update = function()
	 if AddOnCharacterDropDown then AddOnCharacterDropDown:Hide() end
	 AddonCharacterDropDown:Hide()
	 local numAddons = GetNumAddOns()
	 for i=1, MAX_ADDONS_DISPLAYED do
		 local entry = _G["AddonListEntry"..i]
		 local checkbox = _G["AddonListEntry"..i.."Enabled"]
		 local title = _G["AddonListEntry"..i.."Title"]
		 local status = _G["AddonListEntry"..i.."Status"]

		 local lockGlobalIcon = iconsGlobal[i]
		 local lockClassIcon = iconsClass[i]			
		 local memIcon = iconsMemory[i]

		 local addonIndex = entry:GetID()
		 if addonIndex > numAddons then
			 entry.memory = nil
			 memIcon:Hide()
			 lockGlobalIcon:Hide()
			 lockClassIcon:Hide()
			 --checkbox:Show()
		 else
			 local enabled = (GetAddOnEnableState(character, addonIndex) > 0)
			 if enabled then
				 local depsEnabled = CheckAddonDependencies(GetAddOnDependencies(addonIndex))
				 if not depsEnabled then
					title:SetTextColor(1.0, 0.1, 0.1)
					status:SetText(_G["ADDON_DEP_DISABLED"])
				 end
				 if IsAddOnLoadOnDemand(addonIndex) and not IsAddOnLoaded(addonIndex) and depsEnabled then
					AddonList_SetStatus(entry, true, false, false)
				 end
				 local memory = GetAddOnMemoryUsage(addonIndex)
				 entry.memory = memory
				 local usage = memory/8000 -- just needed some baseline!
				 if usage > 0.8 then
					memIcon:SetNormalTexture([[Interface\AddOns\ClassierAddonList\textures\mem5]])
				 elseif usage > 0.6 then
					memIcon:SetNormalTexture([[Interface\AddOns\ClassierAddonList\textures\mem4]])
				 elseif usage > 0.4 then
					memIcon:SetNormalTexture([[Interface\AddOns\ClassierAddonList\textures\mem3]])
				 elseif usage > 0.2 then
					memIcon:SetNormalTexture([[Interface\AddOns\ClassierAddonList\textures\mem2]])
				 elseif usage > 0.1 then
					 memIcon:SetNormalTexture([[Interface\AddOns\ClassierAddonList\textures\mem1]])
				 else
					 memIcon:SetNormalTexture([[Interface\AddOns\ClassierAddonList\textures\mem0]])
				 end
				 memIcon:Show()
			 else
				 entry.memory = nil
				 memIcon:Hide()
			 end
			 local EnabledGlobal = IsAddonEnabledGlobal(addonIndex)
			 lockGlobalIcon:SetShown(EnabledGlobal)
			 local EnabledClass = IsAddonEnabledClass(addonIndex)
			 lockClassIcon:SetShown(EnabledClass)
			 local name, _title, notes, loadable, reason, security = GetAddOnInfo(addonIndex)
			 --enabled = (GetAddOnEnableState(character, addonIndex) > 0)
			 
			 --print(name, _title, notes, enabled, loadable, reason, security)
			 --checkbox:SetChecked(enabled)
			 
			 --if not enabled then --AddonList_SetStatus(entry, x, x, true) --requires reload	 --AddonList_SetStatus(entry, true, x, x) --Load Addon
				 --AddonList_SetStatus(entry, false, true, false)
				 --status:SetText("Disabled")
			 --end
			 
			 --if ((loadable) or ((enabled) and ((reason == "DEP_DEMAND_LOADED") or (reason == "DEMAND_LOADED")))) then
				 --title:SetTextColor(1.0, 0.78, 0.0)
			 --elseif ((enabled) and (reason ~= "DEP_DISABLED")) then
				 --title:SetTextColor(1.0, 0.1, 0.1)
			 --else
				 --title:SetTextColor(0.5, 0.5, 0.5)
			 --end
			 --if (_title) then title:SetText(_title) else title:SetText(name) end
			 
			 local EnabledChar = IsAddonEnabledChar(addonIndex)
			 local EnabledCheck = (not (EnabledGlobal or EnabledClass))
			 if EnabledCheck then
				 --SetAddonEnabledChar(index, checkbox:GetChecked())
				 --checkbox:SetChecked(enabled) --EnabledChar)
			 end
			 checkbox:SetShown(EnabledCheck)
			 --print(_title, "EnabledGlobal",EnabledGlobal,"EnabledClass",EnabledClass,"EnabledChar",EnabledChar)
		 end
	 end
end

local addon = CreateFrame("Frame")
addon:SetScript("OnEvent", function(self, event, ...) self[event](self, ...) end)
addon:RegisterEvent("ADDON_LOADED")
addon:RegisterEvent("PLAYER_LOGIN")
addon:RegisterEvent("PLAYER_LOGOUT")

function addon:Enabler(_character)
	 --UIDropDownMenu_SetSelectedValue(AddonCharacterDropDown, ALL)
	 --DisableAllAddOns(nil)
	 --EnableAddOn(ADDON_NAME, nil)
	 --print("I was here.......", _character)
	 for i=1, GetNumAddOns() do
		 DisableAddOn(i, nil)
		 local name, title, notes, loadable, reason, security = GetAddOnInfo(i)
		 if name == ADDON_NAME or IsAddonEnabled(i) then --and security == "INSECURE"
			 if IsAddonEnabledGlobal(i) then 
				 EnableAddOn(i, nil)
			 else
				 EnableAddOn(i, _character)
			 end
		 end	 
	 end	 
end

local updater = CreateFrame("Frame", nil, AddonList)
updater:SetScript("OnShow", function(self) UpdateAddOnMemoryUsage() if not self.timer then self.timer = C_Timer.NewTicker(10, UpdateAddOnMemoryUsage) end end)
updater:SetScript("OnHide", function(self) self.timer:Cancel() self.timer = nil end)
hooksecurefunc("AddonList_LoadAddOn", function(self) UpdateAddOnMemoryUsage() AddonList_Update() end)
hooksecurefunc("AddonTooltip_Update", function(self) local memory = self.memory if memory then local text = "" if memory > 1000 then memory = memory / 1000 text = L["Memory: %.02f MB"]:format(memory) else text = L["Memory: %.0f KB"]:format(memory) end GameTooltip:AddLine(text) end end)
hooksecurefunc("AddonList_Update", AddonList_Update)
hooksecurefunc("DisableAllAddOns", function(character) addon:Enabler(_character) end)

--[[
GameFontNormal
GameFontNormalSmall
GameFontNormalLarge
GameFontHighlight
GameFontHighlightSmall
GameFontHighlightSmallOutline
GameFontHighlightLarge
GameFontDisable
GameFontDisableSmall
GameFontDisableLarge
GameFontGreen
GameFontGreenSmall
GameFontGreenLarge
GameFontRed
GameFontRedSmall
GameFontRedLarge
GameFontWhite
GameFontDarkGraySmall
NumberFontNormalYellow
NumberFontNormalSmallGray
QuestFontNormalSmall
DialogButtonHighlightText
ErrorFont
TextStatusBarText
CombatLogFont
]]--

local hint1 = AddonList:CreateFontString("AddonList", "ARTWORK", "GameFontNormalLarge")
hint1:SetPoint("TOPLEFT", 20, -23)
hint1:SetWidth(AddonList:GetWidth())
hint1:SetJustifyH("LEFT")
hint1:SetText("|cffc0c0c0Shift+Click to always load.|r")

local hint2 = AddonList:CreateFontString("AddonList", "ARTWORK", "GameFontNormalLarge") --GameFontWhite --NumberFontNormalYellow
hint2:SetPoint("TOPLEFT", 20, -43)
hint2:SetWidth(400)
hint2:SetJustifyH("LEFT")
hint2:SetText("|cffc0c0c0Control+Click to always load for class.|r")


local function OnClick(self, button, down)
	 if button == "LeftButton" then
		 local index = self:GetParent():GetID()
		 if IsShiftKeyDown() then
			 SetAddonEnabledGlobal(index, not IsAddonEnabledGlobal(index))
			 --print("IsAddonEnabledGlobal(index)", IsAddonEnabledGlobal(index))
			 AddonList_Enable(index, true)
 			 --local enabled = (GetAddOnEnableState(character, addonIndex) > 0)
			 --if enabled and not IsAddonEnabled(addonIndex) then
				 --enabled = false
				 --DisableAddOn(addonIndex)
			 --elseif (not enabled) and (IsAddonEnabled(addonIndex)) then
				 --enabled = true
				 --EnableAddOn(name)
			 --end
		 elseif IsControlKeyDown() then
			 SetAddonEnabledClass(index, not IsAddonEnabledClass(index))
			 --print("IsAddonEnabledClass(index)", IsAddonEnabledClass(index))
			 AddonList_Enable(index, true)
		 else
			 SetAddonEnabledChar(index, not IsAddonEnabledChar(index))
			 --print("IsAddonEnabledChar(index)", IsAddonEnabledChar(index))
			 --AddonList_Enable(index, true)
		 end
	 end
end

function addon:ADDON_LOADED(name)
	 if name == ADDON_NAME then 
		 ADDON_LOADED = true
		 self:UnregisterEvent("ADDON_LOADED")

		 character = UnitName("player")
		 characterX = character.."-"..gsub(GetRealmName()," ","")
		 class = UnitClass("player") -- localizedName
		 
		 ClassierAddonListDB.EnabledGlobal = ClassierAddonListDB.EnabledGlobal or {}
		 ClassierAddonListDB.EnabledGlobal[ADDON_NAME] = nil
		 ClassierAddonListDB.EnabledClass = ClassierAddonListDB.EnabledClass or {}
		 ClassierAddonListDB.EnabledClass[class] = ClassierAddonListDB.EnabledClass[class] or {}
		 ClassierAddonListDB.EnabledClass[class][ADDON_NAME] = true
		 ClassierAddonListDB.EnabledChar = ClassierAddonListDB.EnabledChar or {}
		 ClassierAddonListDB.EnabledChar[characterX] = ClassierAddonListDB.EnabledChar[characterX] or {}
		 ClassierAddonListDB.EnabledChar[characterX][ADDON_NAME] = nil
		 ClassierAddonListDB.colors = ClassierAddonListDB.colors or {}
		 local classes = {}
		 FillLocalizedClassList(classes, true)
		 for token, localizedName in pairs(classes) do --local color = RAID_CLASS_COLORS[token]; ChatFrame1:AddMessage(localizedName, color.r, color.g, color.b)
			 ClassierAddonListDB.EnabledClass[localizedName] = ClassierAddonListDB.EnabledClass[localizedName] or {}
			 ClassierAddonListDB.colors[localizedName] = RAID_CLASS_COLORS[token]
		 end
		 
		 color = ClassierAddonListDB.colors[class]
		 
		 local colorcode = "|cff"..string.format("%02x%02x%02x", color.r*255, color.g*255, color.b*255)
		 --print(color.r, color.g, color.b)
		 local hint2text = "|cffc0c0c0Control+Click for "..colorcode..class.."|cffc0c0c0 only."
		 hint2:SetText(hint2text)

		 if ClassierAddonListDB.colors[class] then
			 for i=1, MAX_ADDONS_DISPLAYED do
				 local checkbox = _G["AddonListEntry"..i.."Enabled"]
				 local lockGlobal = CreateFrame("Button", nil, _G["AddonListEntry"..i], nil, i)
				 lockGlobal:SetSize(16, 16)
				 lockGlobal:SetPoint("CENTER", checkbox, "CENTER")
				 lockGlobal:SetNormalTexture([[Interface\Glues\CharacterSelect\Glues-AddOn-Icons]])
				 lockGlobal:GetNormalTexture():SetTexCoord(0, 16/64, 0, 1) -- AddonList_SetSecurityIcon
				 lockGlobal:SetScript("OnClick", OnClick)
				 lockGlobal:Hide()
				 iconsGlobal[i] = lockGlobal
				 local lockClass = CreateFrame("Button", nil, _G["AddonListEntry"..i], nil, i)
				 lockClass:SetSize(20, 20)
				 lockClass:SetPoint("CENTER", checkbox, "CENTER")
				 lockClass:SetNormalTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes")
				 local englishClass = select(2,UnitClass("player"))
				 local coords = CLASS_ICON_TCOORDS[englishClass] -- get the coordinates of the class icon we want
				 --print(unpack(coords))
				 lockClass:GetNormalTexture():SetTexCoord(unpack(coords))
				 lockClass:SetScript("OnClick", OnClick)
				 lockClass:Hide()
				 lockClass:GetNormalTexture():SetVertexColor(color.r, color.g, color.b, 1)
				 iconsClass[i] = lockClass
				 checkbox:HookScript("OnClick", function(self, ...) OnClick(lockGlobal, ...) end)
				 local mem = CreateFrame("Button", nil, _G["AddonListEntry"..i], nil, i)
				 mem:SetSize(6, 32)
				 mem:SetPoint("RIGHT", checkbox, "LEFT", 1, 0)
				 mem:SetNormalTexture([[Interface\AddOns\ClassierAddonList\textures\mem0]])
				 iconsMemory[i] = mem
			 end
		 end

		 -- check Enabled
		 addon:Enabler()
		 --[[
		 local Enabled = {}
		 --for name in next, ClassierAddonListDB.EnabledGlobal do print("Global", name) Enabled[name] = true end
		 --for name in next, ClassierAddonListDB.EnabledClass[class] do print("Class", name) Enabled[name] = true end
		 for i=1, GetNumAddOns() do --for name in next, Enabled do
			 local _name, _title, notes, loadable, reason, security = GetAddOnInfo(i)
			 if ((IsAddonEnabled(i)) or (ADDON_NAME == _name)) then
				 print("Enabling", _name)
				 EnableAddOn(_name)
				 --AddonList_Enable(index, true)
			 else
				print("Disabling", _name)
				DisableAddOn(_name)
			 end
		 end
		 ]]--
	 end
end	

function addon:PLAYER_LOGIN()
	-- make the panel movable
	local mover = CreateFrame("Frame", "ClassierAddonListMover", AddonList)
	mover:EnableMouse(true)
	mover:SetPoint("TOP", AddonList, "TOP", 0, 0)
	mover:SetWidth(500)
	mover:SetHeight(25)
	mover:SetScript("OnMouseDown", function(self) AddonList:StartMoving() end)
	mover:SetScript("OnMouseUp", function(self) AddonList:StopMovingOrSizing() end)
	AddonList:SetMovable(true)
	AddonList:ClearAllPoints()
	AddonList:SetPoint("CENTER")

	-- let the frame overlap over ui frames
	--UIPanelWindows["AddonList"].area = nil

	-- default to showing the player profile
	UIDropDownMenu_SetSelectedValue(AddonCharacterDropDown, character)
	UIDropDownMenu_SetText(AddonCharacterDropDown, character) -- don't show "Custom" ... this dropdown is weird

	SLASH_CLASSIERADDONLIST1 = "/addons"
	SLASH_CLASSIERADDONLIST2 = "/acp" -- muscle memory ;[
	SLASH_CLASSIERADDONLIST3 = "/cal" -- why not
	SlashCmdList["CLASSIERADDONLIST"] = function(input)
		if not input or input:trim() == "" then
			ShowUIPanel(AddonList)
			return
		end

		local command, rest = input:match("^(%S*)%s*(.-)$")
		command = command and command:lower()
		rest = (rest and rest ~= "") and rest:trim() or nil

		if command == "load" then
			if sets[rest] then
				self:LoadSet(rest)
				self:Print(L["Enabled only addons in set %q."]:format(rest))
			else
				self:Print(L["No set named %q."]:format(rest))
			end
		elseif command == "unload" or command == "disable" then
			if sets[rest] then
				self:DisableSet(rest)
				self:Print(L["Disabled addons in set %q."]:format(rest))
			else
				self:Print(L["No set named %q."]:format(rest))
			end
		elseif command == "enable" then
			if sets[rest] then
				self:EnableSet(rest)
				self:Print(L["Enabled addons in set %q."]:format(rest))
			else
				self:Print(L["No set named %q."]:format(rest))
			end
		elseif command == "disableall" then
			DisableAllAddOns(character)
			_G.AddonList_Update()
			self:Print(L["Disabled all addons."])
		elseif command == "reset" then
			self:LoadSet(DEFAULT_SET)
			self:Print(L["Reset addons to what was enabled at login."])
		end
	end

	-- add some reload slashes, too
	SLASH_RELOADUI1 = "/reloadui"
	SLASH_RELOADUI2 = "/rl"
	SlashCmdList["RELOADUI"] = function()
		ReloadUI()
	end
end


